SuperStrict

NoDebug

Rem
bbdoc:	BASIC/BlitzMax Extended runtime
End Rem
Module BRL.Blitz

ModuleInfo "Version: 0.8.16 Alpha"
ModuleInfo "Original Author: Mark Sibly"
ModuleInfo "Author: Fabian Niemann"
ModuleInfo "License: zlib/libpng"
ModuleInfo "Copyright: Blitz Research Ltd"

' Configure compilation options

ModuleInfo "CC_OPTS: -std=gnu99 -O3"

Rem Enables experimental features that improve BlitzMax

	Experimental features include fine tuning for performance, like compile time
	objects. Those are created by hand as C structs and pointed to. That way, no
	slow heap allocation code must be executed.

	Certain functions can also only be available in experimental mode for testing
	purposes. If they are considered stable, they will be included by default.

	When enabled you must make sure that all other external libraries as well as
	your own code are using the new features correctly.

	Default: off
End Rem
'ModuleInfo "CC_OPTS: -DBB_ENABLE_EXPERIMENTAL"


Rem Disables some deprecated features, that might still be used by legacy code
	Default: off
End Rem
'ModuleInfo "CC_OPTS: -DBB_DISABLE_DEPRECATED"


Rem Sets the class register to a constant size buffer

	This can be used to increase execution speed of the application a little, as all
	classes will be registered in a compile time sized buffer. Thus avoiding multple
	possibly unnecessary calls of 'malloc' during run time.
	
	The value this option is set to will be used as the size for the class register.
	
	It's best to use this after the program is complete and ready to ship. This way you
	know the number of classes used in it.
	Note: The value must be at least 10.
	
	If turned off, every 256 classes a new chunk of memory must be allocated dynamically.
	Also all of the previously registered classes must be copied in the process.


	[Off-Topic]
	
	This is something that should totally be done by the compiler. Since it is not,
	the only solution is to find out how many classes are needed by yourself:
	
	Function CountRegisteredTypes:Int()
		Extern "C"
			Function bbObjectRegisteredTypes:Byte Ptr Ptr(count:Int Ptr)
		End Extern
		Local count:Int; bbObjectRegisteredTypes(Varptr count); Return count
	End Function

	Use this function to find out how many classes your application uses.

	[/Off-Topic]

	Default: off
End Rem
'ModuleInfo "CC_OPTS: -DBB_CONST_CLASS_REGISTER=512"


?Debug
 ModuleInfo "CC_OPTS: -DBB_DEBUG"
?
?Threaded
 ModuleInfo "CC_OPTS: -DBB_THREADED"
?

' Core language features

Import "c/blitz_app.c"
Import "c/blitz_types.c"
Import "c/blitz_cclib.c"
Import "c/blitz_unicode.c"
Import "c/blitz_memory.c"
Import "c/blitz_module.c"
Import "c/blitz_class.c"
Import "c/blitz_object.c"
Import "c/blitz_string.c"
Import "c/blitz_array.c"
Import "c/blitz_handle.c"
Import "c/blitz_debug.c"
Import "c/blitz_incbin.c"
Import "c/blitz_thread.c"
Import "c/blitz_ex.c"

?Threaded
 Import "c/blitz_gc_ms.c"
?Not Threaded
 Import "c/blitz_gc_rc.c"
?

Import "c/exceptions/blitz_exception.c"

Import "c/exceptions/blitz_blitzexception.c"
Import "c/exceptions/blitz_nullobjectexception.c"
Import "c/exceptions/blitz_nullmethodexception.c"
Import "c/exceptions/blitz_nullfunctionexception.c"
Import "c/exceptions/blitz_arrayboundsexception.c"
Import "c/exceptions/blitz_outofdataexception.c"
Import "c/exceptions/blitz_runtimeexception.c"

?Win32X86
 Import "asm/blitz_ex.win32.x86.s"
 Import "asm/blitz_gc.win32.x86.s"
 Import "asm/blitz_ftoi.win32.x86.s"
?LinuxX86
 Import "asm/blitz_ex.linux.x86.s"
 Import "asm/blitz_gc.linux.x86.s"
 Import "asm/blitz_ftoi.linux.x86.s"
?MacosX86
 Import "asm/blitz_ex.macos.x86.s"
 Import "asm/blitz_gc.macos.x86.s"
 Import "asm/blitz_ftoi.macos.x86.s"
?MacosPPC
 Import "asm/blitz_ex.macos.ppc.s"
 Import "asm/blitz_gc.macos.ppc.s"
?

Extern
	Global OnDebugStop() = "bbOnDebugStop"
	Global OnDebugLog(message:String) = "bbOnDebugLog"
End Extern

Include "src/types.bmx"

?Ignore
' For documentation purposes only, everything is implemented in C
 Include "src/keywords.bmx"

 Include "src/IEnumerator.bmx"
 Include "src/IEnumerable.bmx"

 Include "src/Object.bmx"
 Include "src/String.bmx"
 Include "src/Array.bmx"

 Include "src/exceptions/TBaseException.bmx"
 Include "src/exceptions/TBlitzException.bmx"
 Include "src/exceptions/TNullObjectException.bmx"
 Include "src/exceptions/TNullMethodException.bmx"
 Include "src/exceptions/TNullFunctionException.bmx"
 Include "src/exceptions/TArrayBoundsException.bmx"
 Include "src/exceptions/TOutOfDataException.bmx"
 Include "src/exceptions/TRuntimeException.bmx"
?

Rem
bbdoc:	Stops program execution and enters the debugger
about:	If there is no debugger present, this command is ignored.
End Rem
Function DebugStop()
	OnDebugStop()
End Function

Rem
bbdoc:	Write a string to debug log
about:	If there is no debugger present, this command is ignored.
End Rem
Function DebugLog(message:String)
	OnDebugLog(message)
End Function

Extern

Rem
bbdoc:	Application directory
about:	The #AppDir global variable contains the fully qualified directory of the currently
		executing application. An application's initial current directory is also set to #AppDir
		when an application starts.
End Rem
Global AppDir:String = "bbAppDir"

Rem
bbdoc:	Application file name
about:	The #AppFile global variable contains the fully qualified file name of the currently
		executing application.
End Rem
Global AppFile:String = "bbAppFile"

Rem
bbdoc:	Application title
about:	The #AppTitle global variable is used by various commands when a
		default application title is required - for example, when opening simple 
		windows or requesters.
		
		<p>Initially, #AppTitle is set the value "BlitzMax Application". However, you may change
		#AppTitle at any time with a simple assignment.</p>
End Rem
Global AppTitle:String = "bbAppTitle"

Rem
bbdoc:	Arguments passed to the application at startup
about:	The #AppArgs global array contains the command line parameters sent to an application
		when it was started. The first element of #AppArgs always contains the name of the 
		application. However, the format of the name may change depending on how the application
		was launched. Use #AppDir or #AppFile for consistent information about the applications name
		or directory.
End Rem
Global AppArgs:String[] = "bbAppArgs"

Rem
bbdoc:	Directory from which application was launched
about:	The #LaunchDir global variable contains the current directory at the time the
		application was launched. This is mostly of use to command line tools which may need to
		access the 'shell' current directory as opposed to the application directory.
End Rem
Global LaunchDir:String = "bbLaunchDir"

Rem
bbdoc:	Add a function to be called when the program ends
about:	OnEnd allows you to specify a function to be called when the program ends.
		The callback functions are called in the reverse order to that in which they were added.
		
		<p class="bbdoc-note">At least 32 callback functions can be registered.</p>
param:	callback A pointer to the funciton that will be called upon exit
End Rem
Function OnEnd(callback()) = "bbOnEnd"

Rem
bbdoc:	Read a string from stdin
returns: A string read from stdin. The newline terminator, if any, is included in the returned string.
End Rem
Function ReadStdIn:String() = "bbReadStdin"

Rem
bbdoc:	Writes a string to stdout
about:	After writing @str to the stdout stream it is flushes.
End Rem
Function WriteStdOut(str:String) = "bbWriteStdout"

Rem
bbdoc:	Writes a string to stderr
about:	After writing @str to the stderr stream it is flushed.
End Rem
Function WriteStdErr(str:String) = "bbWriteStderr"

Rem
bbdoc:	Throws a #TRuntimeException with the given error message.
about:	This can be used to notify other parts of a program of an error
		that could not be resolved at compile-time.
		
		<p class="bbdoc-note">Specific exceptions are a better method of error reporting.</p>
		
		<table class="bbdoc-params">
			<tr><td>message</td><td>A description of the problem</td></tr>
		</table>		
End Rem
Function RuntimeError(message:String) = "brl_blitz_RuntimeError"

Rem
bbdoc:	Wait for a given number of milliseconds
about:	Delay suspends program execution for at least the given number of milliseconds.
		
		<table class="bbdoc-params">
			<tr><td>milliseconds</td><td>The number of milliseconds to suspend the program#s execution.</td></tr>
		</table>
		
		<p class="bbdoc-note">A millisecond is one thousandth of a second.</p>
End Rem
Function Delay(milliseconds:Int) = "bbDelay"

Rem
bbdoc:	Get millisecond counter
returns: Milliseconds since computer turned on.
about:	#MilliSecs returns the number of milliseconds elapsed since the computer
		was turned on.

		<p class="bbdoc-note">A millisecond is one thousandth of a second.</p>
End Rem
Function MilliSecs:Int() = "bbMilliSecs"

Rem
	bbdoc:	Allocates a block of memory
	returns: A pointer to the new block of memory @size bytes long
	about:	<table class="bbdoc-params">
				<tr><td>size</td><td>The size of the memory block</td></tr>
			</table>	
End Rem
Function MemAlloc:Byte Ptr(size:Int) = "bbMemAlloc"

Rem
bbdoc:	Frees allocated memory
about:	The memory specified by @mem must have been previously allocated by #MemAlloc or #MemExtend.

		<table class="bbdoc-params">
			<tr><td>mem</td><td>A pointer to a previously allocated block of memory</td></tr>
		</table>
End Rem
Function MemFree(mem:Byte Ptr) = "bbMemFree"

Rem
bbdoc:	Extends a block of memory
returns: A new block of memory @new_size bytes long
about:	An existing block of memory specified by @mem and @size is copied into a new block
		of memory @new_size bytes long. The existing block is released and the new block is returned.
		
		<table class="bbdoc-params">
			<tr><td>mem</td><td>A pointer to a previously allocated block of memory</td></tr>
			<tr><td>size</td><td>The size of the existing block of memory (pointed to by @mem)</td></tr>
			<tr><td>new_size</td><td>The size of the new block of memory</td></tr>
		</table> 
End Rem
Function MemExtend:Byte Ptr(mem:Byte Ptr, size:Int, new_size:Int) = "bbMemExtend"

Rem
bbdoc:	Clears a block of memory to 0
about:	This fills the given @mem block with the value '0'.

		<table class="bbdoc-params">
			<tr><td>mem</td><td>A pointer to a previously allocated block of memory</td></tr>
			<tr><td>size</td><td>The number of bytes that will be zeroed out (smaller than or equal to the size of the memory block pointed to by @mem).</td></tr>
		</table>
End Rem
Function MemClear(mem:Byte Ptr, size:Int) = "bbMemClear"

Rem
bbdoc:	Copies a non-overlapping block of memory
about:	<table class="bbdoc-params">
			<tr><td>dest</td><td>A pointer to a previously allocated block of memory that @src is copied into</td></tr>
			<tr><td>src</td><td>A pointer to a block of memory containing the values to be copied</td></tr>
			<tr><td>size</td><td>The number of bytes copied from @src to @dest</td></tr>
		</table>
End Rem
Function MemCopy(dest:Byte Ptr, src:Byte Ptr, size:Int) = "bbMemCopy"

Rem
bbdoc:	Copies a potentially overlapping block of memory
End Rem
Function MemMove(dest:Byte Ptr, src:Byte Ptr, size:Int) = "bbMemMove"

Rem
bbdoc:	Sets the garbage collector to automatic operation mode.
about:	This is the default mode.

		<p class="bbdoc-see">Used by #GCSetMode and #GCGetMode</p>
End Rem
Const GC_AUTOMATIC:Int	= 1

Rem
bbdoc:	Sets the garbage collector to manual mode.
about:	<p class="bbdoc-see">Used by #GCSetMode and #GCGetMode</p>
End Rem
Const GC_MANUAL:Int		= 2

' Collect on every allocation - reference counting GC (single threaded) only
Const GC_AGGRESSIVE:Int	= -1

Rem
bbdoc:	Gets the current operating mode of the garbage collector.
returns: The current operating mode.
End Rem
Function GCGetMode:Int() = "bbGCGetMode"

Rem
bbdoc:	Sets the garbage collector's operating mode
about:	@mode can be one of the following:

		<dl>
			<dt>#GC_AUTOMATIC</dt><dd>Memory will be automatically garbage collected</dd>
			<dt>#GC_MANUAL</dt><dd>No memory will be collected until a call to GCCollect is made</dd>
		</dl>

		<p>The default mode is the automatic GC.</p>
End Rem
Function GCSetMode(mode:Int) = "bbGCSetMode"

Rem
bbdoc:	Suspends the garbage collector
about:	<p>This temporarily suspends the garbage collector. No garbage
		collection will be performed following a call to #GCSuspend.</p>
		
		<p>Use #GCResume to resume the garbage collector. Note that #GCSuspend
		and #GCResume 'nest', meaning that each call to #GCSuspend must be 
		matched by a call to #GCResume.</p>
		
		<p>The GC can be suspended for <tt>INT_MAX - 1</tt> times.</p>
End Rem
Function GCSuspend() = "bbGCSuspend"

Rem
bbdoc:	Resumes the garbage collector
about:	<p>#GCResume resumes garbage collection following a call to #GCSuspend.</p>

		<p class="bbdoc-see">See #GCSuspend for more details.</p>
End Rem
Function GCResume() = "bbGCResume"

Rem
bbdoc:	Run garbage collector
returns: The amount of memory in bytes, that was collected.
about:	This function will have no effect if the garbage collector has been
		suspended due to #GCSuspend.
End Rem
Function GCCollect:Int() = "bbGCCollect"

Rem
bbdoc:	Memory allocated by application
returns: The amount of memory, in bytes, currently allocated by the application
about:	This function only returns 'managed memory'. This includes all objects, strings and
		arrays in use by the application.
End Rem
Function GCMemAlloced:Int() = "bbGCMemAlloced"

Rem
bbdoc:	Converts an object to an integer handle
returns: An integer object handle
about:	After converting an object to an integer handle, you must later
		release it using the #Release command.
End Rem
Function HandleFromObject:Int(obj:Object) = "bbHandleFromObject"

Rem
bbdoc:	Converts an integer handle to the associated object
returns: The object associated with the integer handle
End Rem
Function HandleToObject:Object(handle:Int) = "bbHandleToObject"

End Extern
