/**
 * \brief	Commonly used macros and standard libaries
 *
 * This file should be included everywhere, as it provides
 * a range of commonly used macros and libraries as well as the basic
 * functionality needed by every part of a library working with BlitzMax.
 *
 * \author	Fabian Niemann
 */

#ifndef BLITZ_COMMON_H
#define BLITZ_COMMON_H

//	Standard includes

#include <ctype.h>
#include <errno.h>
#include <math.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "blitz_macros.h"

#ifdef BB_OS_WIN32
#	include <windows.h>
#endif

#include "blitz_types.h"

#ifdef BB_THREADED
#	ifndef BB_GC_MS
#		define BB_GC_MS				TRUE
#	endif
#else
#	ifndef BB_GC_RC
#		define BB_GC_RC				TRUE
#	endif
#endif

BB_BEGIN_DECLS

typedef struct	BBByteArray			BBByteArray;
typedef struct	BBShortArray		BBShortArray;
typedef struct	BBIntArray			BBIntArray;
typedef struct	BBLongArray			BBLongArray;
typedef struct	BBFloatArray		BBFloatArray;
typedef struct	BBDoubleArray		BBDoubleArray;
typedef struct	BBObjectArray		BBObjectArray;
typedef struct	BBStringArray		BBStringArray;
typedef struct	BBArrayArray		BBArrayArray;

typedef enum	BBGCAllocFlags		BBGCAllocFlags;
typedef enum	BBGCMode			BBGCMode;
typedef enum	BBGCDebugMode		BBGCDebugMode;

BB_END_DECLS

#endif	// BLITZ_COMMON_H
