/**
 * \brief	Primitive numeric type conversion and handling functions
 *
 * These functions are used by the BlitzMax compiler to handle
 * primitive numeric types. They are relatively useless in the
 * context of C/C++ modules or libraries. Try not to use them
 * manually from said context.
 *
 * \author	Mark Sibly
 */

#ifndef BLITZ_CCLIB_H
#define BLITZ_CCLIB_H

#include "_common.h"

BB_BEGIN_DECLS

BBInt bbIntAbs(BBInt x);
BBInt bbIntSgn(BBInt x);
BBInt bbIntMod(BBInt x, BBInt y);
BBInt bbIntMin(BBInt x, BBInt y);
BBInt bbIntMax(BBInt x, BBInt y);
void  bbIntToLong(BBLong *result, BBInt x);

BBDouble bbFloatAbs(BBDouble x);
BBDouble bbFloatSgn(BBDouble x);
BBDouble bbFloatPow(BBDouble x, BBDouble y);
BBDouble bbFloatMod(BBDouble x, BBDouble y);
BBDouble bbFloatMin(BBDouble x, BBDouble y);
BBDouble bbFloatMax(BBDouble x, BBDouble y);
BBInt bbFloatToInt(BBDouble x);
void bbFloatToLong(BBLong *result, BBDouble x);

void bbLongNeg(BBLong *result, BBLong x);
void bbLongNot(BBLong *result, BBLong x);
void bbLongAbs(BBLong *result, BBLong x);
void bbLongSgn(BBLong *result, BBLong x);
void bbLongAdd(BBLong *result, BBLong x, BBLong y);
void bbLongSub(BBLong *result, BBLong x, BBLong y);
void bbLongMul(BBLong *result, BBLong x, BBLong y);
void bbLongDiv(BBLong *result, BBLong x, BBLong y);
void bbLongMod(BBLong *result, BBLong x, BBLong y);
void bbLongMin(BBLong *result, BBLong x, BBLong y);
void bbLongMax(BBLong *result, BBLong x, BBLong y);
void bbLongAnd(BBLong *result, BBLong x, BBLong y);
void bbLongOrl(BBLong *result, BBLong x, BBLong y);
void bbLongXor(BBLong *result, BBLong x, BBLong y);
void bbLongShl(BBLong *result, BBLong x, BBLong y);
void bbLongShr(BBLong *result, BBLong x, BBLong y);
void bbLongSar(BBLong *result, BBLong x, BBLong y);
BBInt bbLongSlt(BBLong x, BBLong y);
BBInt bbLongSgt(BBLong x, BBLong y);
BBInt bbLongSle(BBLong x, BBLong y);
BBInt bbLongSge(BBLong x, BBLong y);
BBInt bbLongSeq(BBLong x, BBLong y);
BBInt bbLongSne(BBLong x, BBLong y);
BBDouble bbLongToFloat(BBLong x);

BB_END_DECLS

#endif	// BLITZ_CCLIB_H
