#include "blitz_class.h"

#ifdef BB_CONST_CLASS_REGISTER

#	if BB_CONST_CLASS_REGISTER > 2147483647 // todo: Why does 'BBCLASS_MAX_NUM_REGISTERED' not work?
#		error "Class register size is too large."
#	elif BB_CONST_CLASS_REGISTER < 10
#		error "Class register size is too small."
#	endif	// end: Validation of class register size

static BBClass* Register[BB_CONST_CLASS_REGISTER];
static BBSize	NumRegistered;

void bbClassRegister(const BBClass *clas)
{
	BB_ASSERT(NumRegistered < BB_CONST_CLASS_REGISTER,
			  "Too many registered classes (max = %i)", BB_CONST_CLASS_REGISTER);
	BB_ASSERT(clas->instance_size < BBCLASS_MAX_INSTANCESIZE,
			  "Class instance size too large");
	
	Register[NumRegistered++] = clas;
}

BBClass** bbClassAllRegistered(BBInt *count)
{
	*count = NumRegistered;
	return Register;
}

void bb_class_register_multiple(BBSize count, ...)
{
	BB_ASSERT(NumRegistered + count < BB_CONST_CLASS_REGISTER,
			  "Too many registered classes (max = %i)", BB_CONST_CLASS_REGISTER);
	
	va_list args;
	
	const BBClass *clas;
	
	va_start(args, count);
	
	for (uint_fast32_t c = 0; c < count; ++c)
	{
		clas = va_arg(args, BBClass*);
		
		BB_ASSERT(clas->instance_size < BBCLASS_MAX_INSTANCESIZE,
				  "Class \"%s\" instance size too large", clas->debug_scope->name);
		
		Register[NumRegistered++] = clas;
	}

	va_end(args);
}

#else	// end: BB_CONST_CLASS_REGISTER

#	define REGISTER_GROW					256

static BBClass **RegisterBase;
static BBClass **RegisterPut;
static BBClass **RegisterEnd;

#	ifdef BB_DEBUG
static BBSize	NumRegistered_debug;
#	endif

void bbClassRegister(const BBClass *clas)
{
	BB_ASSERT(NumRegistered_debug < BBCLASS_MAX_NUM_REGISTERED,
			  "Too many registered classes (max = %i)", BBCLASS_MAX_NUM_REGISTERED);
	BB_ASSERT(clas->instance_size < BBCLASS_MAX_INSTANCESIZE,
			  "Class instance size too large");
	
	if (RegisterPut == RegisterEnd)
	{
		ptrdiff_t len  = RegisterPut - RegisterBase;
		size_t new_len = len + REGISTER_GROW;
		
		RegisterBase = bbMemExtend(RegisterBase, len * sizeof (BBClass*),
								   new_len * sizeof (BBClass*));
		RegisterEnd = RegisterBase + new_len;
		RegisterPut = RegisterBase + len;
	}
	
	*RegisterPut++ = clas;
	
#	ifdef BB_DEBUG
	++NumRegistered_debug;
#	endif
}

BBClass** bbClassAllRegistered(BBInt *count)
{
	*count = RegisterPut - RegisterBase;
	return RegisterBase;
}

void bb_class_register_multiple(BBSize count, ...)
{
	BB_ASSERT(NumRegistered_debug + count < BBCLASS_MAX_NUM_REGISTERED,
			  "Too many registered classes (max = %i)", BBCLASS_MAX_NUM_REGISTERED);
	
	if (RegisterEnd - RegisterPut < count)
	{
		ptrdiff_t len  = RegisterPut - RegisterBase;
		size_t new_len = len + REGISTER_GROW;
		
		RegisterBase = bbMemExtend(RegisterBase, len * sizeof (BBClass*),
								   new_len * sizeof (BBClass*));
		RegisterEnd = RegisterBase + new_len;
		RegisterPut = RegisterBase + len;
	}	
	
	va_list args;
	
	const BBClass *clas;
	
	va_start(args, count);
	
	for (uint_fast32_t c = 0; c < count; ++c)
	{
		clas = va_arg(args, BBClass*);
		
		BB_ASSERT(clas->instance_size < BBCLASS_MAX_INSTANCESIZE,
				  "Class \"%s\" instance size too large", clas->debug_scope->name);
		*RegisterPut++ = clas;
	}

	va_end(args);
	
#	ifdef BB_DEBUG
	NumRegistered_debug += count;
#	endif
}

#endif	// !BB_CONST_CLASS_BUFFER
