
#include "blitz_debug.h"
#include "blitz_object.h"
#include "blitz_string.h"

#ifdef BB_DEBUG

#include "exceptions/blitz_runtimeexception.h"

void bb_assert(const char *expr, const char *file, int line, const char *msg, ...)
{
	size_t cause_len = 64 + strlen(expr) + strlen(file);
	size_t message_len = strlen(msg);
	
	// Contains the failed expression and its position (always thrown)
	char cause_buf[cause_len];
	
	size_t fmt_len;
	fmt_len = snprintf(cause_buf, cause_len,
					   "[ASSERT FAILED] for \"%s\" in \"%s\" line %i", expr, file, line);
	
	// Should actually never happen, included just in case (it's only debug mode)
	if (fmt_len < 0 || fmt_len >= cause_len) {
		snprintf(cause_buf, cause_len, "[ASSERT FAILED]");
	}
	
	BBString *AssertMessage;
	
	if (message_len)
	{
		message_len += 64;
		
		// Contains a message describing the problem in more detail,
		// including the optional variable arguments.
		char message_buf[message_len];
		
		va_list args;
		va_start(args, msg);
		vsnprintf(message_buf, message_len, msg, args);
		va_end(args);
		
		size_t tmp_len = strlen(cause_buf) + strlen(message_buf) + 3;
		char tmp_buf[tmp_len];
		
		strcat(tmp_buf, cause_buf);
		
		// Inserting a line break would result in cutting of the message
		// when displayed by MaxIDE in debug mode.
		strcat(tmp_buf, ": ");
		strcat(tmp_buf, message_buf);
		
		AssertMessage = bbStringFromCString(tmp_buf);
	} else {
		AssertMessage = bbStringFromCString(cause_buf);
	}
		
	bbRuntimeError(AssertMessage);
}

#endif	// BB_DEBUG

static void debug_nop() {}

static void exception_debug_unhandled(BBObject *ex)
{
	bbWriteStderr(ex->clas->ToString(ex));
	exit(-1);
}

void (*bbOnDebugStop)() = debug_nop;
void (*bbOnDebugLog)(BBString *msg) = debug_nop;
void (*bbOnDebugEnterStm)(BBDebugStm *stm) = debug_nop;
void (*bbOnDebugEnterScope)(BBDebugScope *scope, void *inst) = debug_nop;
void (*bbOnDebugLeaveScope)() = debug_nop;
void (*bbOnDebugPushExState)()= debug_nop;
void (*bbOnDebugPopExState)() = debug_nop;

void (*bbOnDebugUnhandledEx)(BBObject *ex) = exception_debug_unhandled;

#ifndef BB_DISABLE_DEPRECATED

void bbCAssertEx()
{
	bbExThrowCString("C Assert failed");
}

#endif	// !BB_DISABLE_DEPRECATED