
#include "blitz_object.h"
#include "blitz_string.h"

#define EX_GROW		10

typedef struct BBExEnv
{
	char			_cpu_state[256]; // worse case - bit ugly, but better than an #ifdef
} BBExEnv;

typedef struct BBExStack
{
	BBExEnv			*ex_base;
	BBExEnv			*ex_sp;
	BBExEnv			*ex_end;
} BBExStack;

#ifdef BB_OS_WIN32

static DWORD exKey()
{
	static int done;
	static DWORD key;
	
	if (!done) {
		key = TlsAlloc();
		done= 1;
	}
	
	return key;
}

BBExStack* getExStack()
{
	return (BBExStack *)TlsGetValue(exKey());
}

void setExStack(BBExStack *stack)
{
	TlsSetValue(exKey(), stack);
}

#else	// BB_OS_WIN32

#include <pthread.h>

static pthread_key_t exKey()
{
	static bool is_done;
	static pthread_key_t key;
	
	if (!is_done) {
		pthread_key_create(&key, 0);
		is_done = true;
	}
	
	return key;
}

BBExStack* getExStack()
{
	return (BBExStack *)pthread_getspecific(exKey());
}

void setExStack(BBExStack *stack)
{
	pthread_setspecific(exKey(), stack);
}

#endif	// !BB_OS_WIN32

static BBExStack *exStack()
{
	BBExStack *stack = getExStack();
	
	if (!stack) {
		stack = bbMemAlloc(sizeof (BBExStack));
		memset(stack, 0, sizeof (BBExStack));
		setExStack(stack);
	}
	
	return stack;
}

static void freeExStack(BBExStack *stack)
{
	bbMemFree(stack->ex_base);
	bbMemFree(stack);
	setExStack(0);
}

void* bbExEnter()
{
	BBExStack *stack = exStack();

	if (stack->ex_sp == stack->ex_end)
	{
		size_t len		= stack->ex_sp - stack->ex_base;
		size_t new_len	= len + EX_GROW;
		
		stack->ex_base	= bbMemExtend(
							stack->ex_base,
							len * sizeof (BBExEnv),
							new_len * sizeof (BBExEnv)
						  );
		
		stack->ex_end	= stack->ex_base + new_len;
		stack->ex_sp	= stack->ex_base + len;
	}

	return (stack->ex_sp++)->_cpu_state;
}

void bbExThrow(BBObject *obj)
{
	BBExStack *stack = getExStack();
	
	if (!stack) {
		bbOnDebugUnhandledEx(obj);
	}
		
	if (--stack->ex_sp == stack->ex_base)
	{
		static char buf[256];
		memcpy(buf, stack->ex_sp->_cpu_state, 256);
		freeExStack(stack);
		_bbExThrow(buf, obj);
	} else {
		_bbExThrow(stack->ex_sp->_cpu_state, obj);
	}
}

void bbExLeave()
{
	BBExStack *stack = getExStack();
	
	// invariant...leaving a Try!
	// assert( st && st->ex_sp!=st->ex_base );
	
	if (--stack->ex_sp == stack->ex_base) {
		freeExStack(stack);
	}
}

void bbExThrowCString(const char *c_str)
{
	bbExThrow((BBObject *)bbStringFromCString(c_str));
}
