#include "blitz_handle.h"
#include "blitz_object.h"

#define HASH_SIZE		1024

#define HASH_SLOT(X)	(((X) / 8) & (HASH_SIZE - 1))	// divide-by-8 for better void* mapping.

static int HandleId;

typedef struct Hash		Hash;

struct Hash
{
	Hash	*succ;
	int		key;
	int		value;
};

static Hash* ObjectHashes[HASH_SIZE];
static Hash* HandleHashes[HASH_SIZE];

static int hash_find(Hash **table, int key)
{
	Hash *hash;
	const int hash_key = HASH_SLOT(key);
	
	for (Hash **p = &table[hash_key]; (hash = *p) && hash->key != key; p = &hash->succ) {}
	return hash ? hash->value : 0;
}

static int hash_remove(Hash **table, int key)
{
	Hash *hash;
	int t_key = HASH_SLOT(key);

	Hash **p;	
	for (p = &table[t_key]; (hash = *p) && key != hash->key; p = &hash->succ) {}
	
	if (!hash) {
		return 0;
	}
	
	int value = hash->value;
	*p = hash->succ;
	bbMemFree(hash);
	
	return value;
}

static void hash_insert(Hash **table, int key, int value)
{
	int slot = HASH_SLOT(key);
	Hash *hash = (Hash *)bbMemAlloc(sizeof (Hash));
	
	hash->key	= key;
	hash->value	= value;
	hash->succ	= table[slot];
	
	table[slot] = hash;
}

//	API //

BBInt bbHandleFromObject(BBObject *obj)
{
	if (obj == &bbNullObject) {
		return 0;
	}
	
	int n = hash_find(ObjectHashes, (int)obj);
	if (n) {
		return n / 8;
	}
		
	BB_RETAIN(obj);
	HandleId += 8;
	
	if (!(HandleId / 8)) {
		HandleId += 8;	// just-in-case!
	}
	
	hash_insert(ObjectHashes, (int)obj, HandleId);
	hash_insert(HandleHashes, HandleId, (int)obj);
	
	return HandleId / 8;
}

BBObject* bbHandleToObject(BBInt handle)
{
	BBObject *obj = (BBObject *)hash_find(HandleHashes, handle * 8);
	return obj ? obj : &bbNullObject;
}

void bbHandleRelease(BBInt handle)
{
	BBObject *obj = (BBObject *)hash_remove(HandleHashes, handle * 8);
	if (!obj) {
		return;
	}
	
	hash_remove(ObjectHashes, (int)obj);
	BB_RELEASE(obj);
}
