
#include "blitz_incbin.h"
#include "blitz_string.h"

typedef struct BBIncbin BBIncbin;

struct BBIncbin
{
	BBIncbin	*succ;
	BBString	*path;
	void		*ptr;
	BBSize		length;
};

static BBIncbin *Incbins;

static BBIncbin* incbin_by_path(const BBString * const path)
{
	for (const BBIncbin *inc = Incbins; inc; inc = inc->succ)
	{
		if (bbStringEquals(path, inc->path)) {
			return inc;
		}
	}
	
	return NULL;
}

//	API

BBInt bbIncbinAdd(BBString *path, const void *ptr, BBSize size)
{
	BBIncbin *inc = incbin_by_path(path);
	if (inc) {
		return 0;
	}
	
	BB_ASSERT(size <= BBINCBIN_MAX_SIZE, "File is too large to be incbin'd.");
	
	inc = (BBIncbin *)bbMemAlloc(sizeof (BBIncbin));
	
	BB_RETAIN(path);
	inc->path	= path;
	inc->ptr	= ptr;
	inc->length	= size;
	inc->succ	= Incbins;
	
	Incbins = inc;

	return 1;
}

void* bbIncbinPtr(const BBString *path)
{
	BBIncbin *inc = incbin_by_path(path);
	return inc ? inc->ptr : NULL;
}

BBSize bbIncbinLen(const BBString *path)
{
	BBIncbin *inc = incbin_by_path(path);
	return inc ? inc->length : 0;
}
