#include "blitz_debug.h"
#include "blitz_memory.h"
#include "blitz_gc.h"

void* bbMemAlloc(BBInt size)
{
	BB_ASSERT(size < BBMEM_MAX_ALLOCSIZE, "Requested size exceeds the maximum.");
	
	size += BBMEM_ALIGNSIZE + 4;
	
	void *data_ptr = malloc(size);
	
	if (!data_ptr) 
	{
		bbGCCollect();
		data_ptr = malloc(size);
		
		if (!data_ptr) {
			return NULL;
		}
	}
	
	void *ptr = (void*)(((unsigned)data_ptr + BBMEM_ALIGNMASK + 4) & ~BBMEM_ALIGNMASK);
	*((void**)ptr - 1) = data_ptr;
	
	return ptr;
}

void bbMemFree(void *mem)
{
	if (mem) {
		free(((void **)mem)[-1]);
	}
}

void* bbMemExtend(void *mem, BBInt size, BBInt new_size)
{
	void *ptr = bbMemAlloc(new_size);
	memcpy(ptr, mem, size);
	
	if (mem) {
		free(((void**)mem)[-1]);
	}
	
	return ptr;
}

void bbMemClear(void *dst, BBInt size)
{
	memset(dst, 0, size);
}

void bbMemCopy(void *dst, const void *src, BBInt size)
{	
	memcpy(dst, src, size);
}

void bbMemMove(void *dst, const void *src, BBInt size)
{
	memmove(dst, src, size);
}
