/**
 * \brief	Interface for using modules
 *
 * *ATTENTION*
 *
 * Importing modules is done totally automatic. You do not have to and
 * propably also cannot call these function manually.
 *
 * \author	Mark Sibly
 */

#ifndef BLITZ_MODULE_H
#define BLITZ_MODULE_H

#include "_common.h"

BB_BEGIN_DECLS

typedef struct BBModule BBModule;

/** Represents a program module
 *
 */
struct BBModule
{
	/** The next module in the list */
	BBModule*	succ;
	
	/** The identity of the module (name and scope) */
	const char*	ident;
	
	/** The entry function of the module */
	void		(*entry)();
};

/** Imports a module an runs its entry function
 *
 * Every module can only be imported once, so if the given module is already
 * imported this funtion will return without doing anything.
 *
 * \param	mod The module to import
 */
void bbModuleImport(BBModule *mod);

/** Registers a module with the system
 *
 * The given module is added to the internal list of modules.
 *
 * \warning	This does no checking whatsoever.
 *
 * \param	mod The module to register
 */
void bbModuleRegister(BBModule *mod);

BB_END_DECLS

#endif	// BLITZ_MODULE_H
