
#include "blitz_types.h"

#include "blitz_object.h"
#include "blitz_string.h"
#include "blitz_array.h"
#include "exceptions/blitz_nullfunctionexception.h"

const char *bbVoidTypeTag	= "?";
const char *bbByteTypeTag	= "b";
const char *bbShortTypeTag	= "s";
const char *bbIntTypeTag	= "i";
const char *bbFloatTypeTag	= "f";
const char *bbDoubleTypeTag	= "d";

const char *bbBytePtrTypeTag= "*b";

const char *bbObjectTypeTag	= ":Object";
const char *bbStringTypeTag	= "$";

// Maps the first character of each valid type tag to a type key
const BBTypeKey bbToTypeKeyData[] =
{
	['?'] = BBTYPEKEY_VOID,
	['b'] = BBTYPEKEY_BYTE,
	['s'] = BBTYPEKEY_SHORT,
	['i'] = BBTYPEKEY_INT,
	['l'] = BBTYPEKEY_LONG,
	['f'] = BBTYPEKEY_FLOAT,
	['d'] = BBTYPEKEY_DOUBLE,
	
	['*'] = BBTYPEKEY_POINTER,
	['('] = BBTYPEKEY_FUNCTION,
	
	['$'] = BBTYPEKEY_STRING,
	[':'] = BBTYPEKEY_OBJECT,
	['['] = BBTYPEKEY_ARRAY
};

const BBTypeKeyInfo bbTypeKeyInfo[] =
{
	[BBTYPEKEY_BYTE] = {
		.size	= sizeof (BBByte),
		.init	= (void *)0,
		.ident	= "Byte"
	},
	
	[BBTYPEKEY_SHORT] = {
		.size	= sizeof (BBShort),
		.init	= (void *)0,
		.ident	= "Short"
	},
	
	[BBTYPEKEY_INT] = {
		.size	= sizeof (BBInt),
		.init	= (void *)0,
		.ident	= "Int"
	},
	
	[BBTYPEKEY_LONG] = {
		.size	= sizeof (BBLong),
		.init	= (void *)0,
		.ident	= "Long"
	},
	
	[BBTYPEKEY_FLOAT] = {
		.size	= sizeof (BBFloat),
		.init	= (void *)0,
		.ident	= "Float"
	},
	
	[BBTYPEKEY_DOUBLE] = {
		.size	= sizeof (BBDouble),
		.init	= (void *)0,
		.ident	= "Double"
	},
	
	[BBTYPEKEY_POINTER]	= {
		.size	= sizeof (void*),
		.init	= NULL,
		.ident	= "Byte Ptr"
	},
	
	[BBTYPEKEY_FUNCTION] = {
		.size	= sizeof (void* (*)),
		.init	= (void *)&brl_blitz_NullFunctionError,
		.ident	= "Function"
	},
	
	//	Complex
	
	[BBTYPEKEY_OBJECT] = {
		.size	= sizeof (BBObject*),
		.init	= (void *)&bbNullObject,
		.ident	= "Object"
	},
	
	[BBTYPEKEY_STRING] = {
		.size	= sizeof (BBString*),
		.init	= (void *)&bbEmptyString,
		.ident	= "String"
	},
	
	[BBTYPEKEY_ARRAY] = {
		.size	= sizeof (BBArray*),
		.init	= (void *)&bbEmptyArray,
		.ident	= "Array"
	}
};

BBInt bbConvertToInt(const void *value, const char *tag)
{
	switch (tag[0])
	{
	case 'b':
		return *(BBByte *)value;
	case 's':
		return *(BBShort *)value;
	case 'i':
		return *(BBInt *)value;
	case 'f':
		return *(BBFloat *)value;
	case 'd':
		return *(BBDouble *)value;
	case '$':
		return bbStringToInt((BBString *)value);
	}
	
	return 0;
}

BBDouble bbConvertToFloat(const void *value, const char *tag)
{
	switch (tag[0])
	{
	case 'b':
		return *(BBByte *)value;
	case 's':
		return *(BBShort *)value;
	case 'i':
		return *(BBInt *)value;
	case 'f':
		return *(BBFloat *)value;
	case 'd':
		return *(BBDouble *)value;
	case '$':
		return bbStringToFloat((BBString *)value);
	}
	
	return 0.0;
}

BBString* bbConvertToString(const void *value, const char *tag)
{
	switch (tag[0])
	{
	case 'b':
		return bbStringFromInt(*(BBByte *)value);
	case 's':
		return bbStringFromInt(*(BBShort *)value);
	case 'i':
		return bbStringFromInt(*(BBInt *)value);
	case 'f':
		return bbStringFromFloat(*(BBFloat *)value);
	case 'd':
		return bbStringFromFloat(*(BBDouble *)value);
	case '$':
		return (BBString *)value;
	}
	
	return 0;
}
