/**
 * \file	blitz_blitzexception.h
 * \brief	Internal language exception
 *
 * The BlitzMax runtime uses its own set of exceptions to signal errors.
 * These exception were originally implemented in BlitzMax and have been ported
 * to C in order to increase speed and improve access from C/C++.
 *
 * \author	Fabian Niemann
 */

#ifndef BLITZ_EX_BLITZEXCEPTION_H
#define BLITZ_EX_BLITZEXCEPTION_H

#include "blitz_exception.h"

BB_BEGIN_DECLS

/** Base object for all of BlitzMax' intern exceptions
 *
 * 
 *
 * \extends	BBException
 */
struct BBBlitzException
{
	/** The intern BlitzMax exception's class
	 * 
	 * This should always point to a class extending bbBlitzExceptionClass.
	 */
	BBClass			*clas;
	int				refs;
	
	BBException		*cause;
};

#define BBBLITZEXCEPTION_ALLOC(clas) \
	({																			\
		(BBBlitzException *)													\
		bbGCAllocObject(														\
			sizeof (BBBlitzException),											\
			(clas),																\
			BBGC_FINALIZE														\
		);																		\
	})

extern BBClass bbBlitzExceptionClass;

BB_END_DECLS

#endif	// BLITZ_EX_BLITZEXCEPTION_H
