#include "blitz_exception.h"
#include "../../blitz.h"

static const bb_debugscope_t ClassScope =
{
	.kind	= BBDEBUGSCOPE_USERTYPE,
	.name	= "TBaseException",
	.decls	=
	{
		[0] = {
			BBDEBUGDECL_TYPEMETHOD,
			"initCause", "(:TBaseException)i",
			BBCLASS_VFN_OFFSETOF(__BBEXCEPTION_VFN_INITCAUSE__)
		},
			
		[1] = {
			BBDEBUGDECL_TYPEMETHOD,
			"getCause", "():TBaseException",
			BBCLASS_VFN_OFFSETOF(__BBEXCEPTION_VFN_GETCAUSE__)
		},
		
		BBDEBUGDECL_END
	}
};

BBClass bbExceptionClass =
{
	.super			= &bbObjectClass,
	.free			= bbObjectFree,
	.debug_scope	= &ClassScope,
	.instance_size	= sizeof (BBException),
	
	.ctor			= bbObjectCtor,
	.dtor			= bbExceptionDtor,
	.ToString		= bbObjectToString,
	.Compare		= bbObjectCompare,
	.SendMessage	= bbObjectSendMessage,
	.Clone			= bbObjectClone,
	
	.vfns =
	{
		[__BBEXCEPTION_VFN_INITCAUSE__]	= bbExceptionInitCause,
		[__BBEXCEPTION_VFN_GETCAUSE__]	= bbExceptionGetCause
	}
};

void bbExceptionCtor(BBException *ex)
{
	ex->clas		= &bbExceptionClass;
	ex->cause		= &bbNullObject;
}

void bbExceptionDtor(BBException *ex)
{
	ex->clas		= NULL;
	BB_RELEASE(ex->cause);
}

BBInt bbExceptionCompare(const BBException *ex, const BBException *rhs)
{
	if (ex == rhs) {
		return 0;
	}
	
	if (ex->clas != rhs->clas) {
		return 1;
	}

	if (ex->cause != rhs->cause) {
		return 2;
	}
	
	//	same exceptions
	return 0;
}

BBException* bbExceptionClone(BBException *ex)
{
	BBException *out = bbGCAllocObject(sizeof (BBException), ex->clas, BBGC_DEFAULT);
	
	BB_RETAIN(ex->cause);
	out->cause		= ex->cause;
	
	return out;
}

void bbExceptionInitCause(BBException *ex, BBException *cause)
{
	if (ex->cause == BBNULL && cause != ex) {
		BB_RETAIN(cause);
		ex->cause	= cause;
	}
}

BBException* bbExceptionGetCause(const BBException *ex)
{
	return ex->cause;
}
