/**
 * \file	blitz_nullfunctionexception.h
 * \brief	Exception concerning calls of invalid function pointers
 *
 * \author	Fabian Niemann
 */

#ifndef BLITZ_EX_NULLFUNCTIONEXCEPTION_H
#define BLITZ_EX_NULLFUNCTIONEXCEPTION_H

#include "blitz_blitzexception.h"

BB_BEGIN_DECLS

/** Thrown in case of an attempt to call an uninitialized function pointer
 *
 * \extends	BBBlitzException
 */
struct BBNullFunctionException
{
	BBClass			*clas;
	int				refs;
	
	BBException		*cause;
};

extern BBClass bbNullFunctionExceptionClass;

/**
 *
 * \returns	"Attempt to call uninitialized function pointer"
 */
BBString* bbNullFunctionExceptionToString(const BBNullFunctionException *ex);

//	API

/** Creates a new NullFunction exception
 *
 */
BBNullFunctionException* bbNullFunctionExceptionCreate();

#define bbNullFunctionError		brl_blitz_NullFunctionError

/** Throws a BBNullFunctionException
 *
 */
void bbNullFunctionError();

BB_END_DECLS

#endif	// BLITZ_EX_NULLFUNCTIONEXCEPTION_H
