/**
 * \file	blitz_nullobjectexception.h
 * \brief	Exception concerning the use of uninitialized (NULL) objects
 *
 * \author	Fabian Niemann
 */

#ifndef BLITZ_EX_NULLOBJECTEXCEPTION_H
#define BLITZ_EX_NULLOBJECTEXCEPTION_H

#include "blitz_blitzexception.h"

BB_BEGIN_DECLS

/** Thrown in case of an attempt to use an uninitialized object
 *
 * \extends	BBBlitzException
 */
struct BBNullObjectException
{
	/** The NullObject exception' class
	 *
	 * This should always point to bbNullObjectExceptionClass
	 */
	BBClass			*clas;
	
	int				refs;
	
	BBException		*cause;
};

extern BBClass bbNullObjectExceptionClass;

/** Returns the error message
 *
 * \returns	"Attempt to access field or method of Null object"
 */
BBString* bbNullObjectExceptionToString(const BBNullObjectException *ex);

//	API

/** Creates a new NullObject exception
 *
 * \returns	A new BBNullObjectException object
 */
BBNullObjectException* bbNullObjectExceptionCreate();

//	Needed to be binary compatible with old modules
#define bbNullObjectError		brl_blitz_NullObjectError

/** Throws a BBNullObjectException
 *
 * This throws a newly created BBNullObjectException.
 * Equivalent to
 *
 * \code
 * bbExThrow(bbNullObjectExceptionCreate());
 * \endcode
 */
void bbNullObjectError();

BB_END_DECLS

#endif	// BLITZ_EX_NULLOBJECTEXCEPTION_H
