#include "blitz_runtimeexception.h"

static const bb_debugscope_t ClassScope =
{
	.kind	= BBDEBUGSCOPE_USERTYPE,
	.name	= "TRuntimeException",
	.decls	=
	{
		[0] = {
			BBDEBUGDECL_TYPEMETHOD,
			"getMessage", "()$",
			BBCLASS_VFN_OFFSETOF(2)
		},
		
		[1] = {
			BBDEBUGDECL_TYPEFUNCTION,
			"Create",
			"($,:TBaseException):TRuntimeException",
			BBCLASS_VFN_OFFSETOF(3)
		},
		
		BBDEBUGDECL_END
	}
};
 
BBClass bbRuntimeExceptionClass =
{
	.super			= &bbExceptionClass,
	.free			= bbObjectFree,
	.debug_scope	= &ClassScope,
	.instance_size	= sizeof (BBRuntimeException),
	
	.ctor			= bbRuntimeExceptionCtor,
	.dtor			= bbRuntimeExceptionDtor,
	
	.ToString		= bbRuntimeExceptionToString,
	.Compare		= bbExceptionCompare,
	.SendMessage	= bbObjectSendMessage,
	.Clone			= bbRuntimeExceptionClone,
	
	.vfns =
	{
		[__BBEXCEPTION_VFN_INITCAUSE__]	= bbExceptionInitCause,
		[__BBEXCEPTION_VFN_GETCAUSE__]	= bbExceptionGetCause,
		bbRuntimeExceptionGetMessage,
		
		bbRuntimeExceptionCreate
	}
};

void bbRuntimeExceptionCtor(BBRuntimeException *ex)
{
	ex->clas	= &bbRuntimeExceptionClass;
	ex->message	= BBNULLSTRING;
	ex->cause	= BBNULL;
}

void bbRuntimeExceptionDtor(BBRuntimeException *ex)
{
	ex->clas	= NULL;
	BB_RELEASE(ex->cause);
	ex->cause	= BBNULL;
	BB_RELEASE(ex->message);
	ex->message	= BBNULLSTRING;
}

BBString* bbRuntimeExceptionToString(const BBRuntimeException *ex)
{
	return ex->message;
}

BBRuntimeException* bbRuntimeExceptionClone(const BBRuntimeException *ex)
{
	BBRuntimeException *copy = BBBLITZEXCEPTION_ALLOC(&bbRuntimeExceptionClass);
	
	copy->cause		= ex->cause;
	BB_RETAIN(copy->cause);
	
	copy->message	= ex->message;
	BB_RETAIN(copy->message);
	
	return copy;
}

//	API

BBRuntimeException* bbRuntimeExceptionCreate(BBString *message, BBException *cause)
{
	BBRuntimeException *ex = BBBLITZEXCEPTION_ALLOC(&bbRuntimeExceptionClass);

	BB_RETAIN(cause);
	ex->cause	= cause;
	
	BB_RETAIN(message);
	ex->message	= message;
	
	return ex;
}

BBString* bbRuntimeExceptionGetMessage(const BBRuntimeException *ex)
{
	return ex->message;
}

void brl_blitz_RuntimeError(BBString *message)
{
	bbExThrow(bbRuntimeExceptionCreate(message, BBNULL));
}