/**
 * \file	blitz_runtimeexception.h
 * \brief	Exception concerning errors notices at runtime
 *
 * \author	Fabian Niemann
 */

#ifndef BLITZ_EX_RUNTIMEEXCEPTION_H
#define BLITZ_EX_RUNTIMEEXCEPTION_H

#include "blitz_blitzexception.h"

BB_BEGIN_DECLS

/** Thrown in case an exceptional event is discovered at runtime
 *
 * \extends	BBException
 */
struct BBRuntimeException
{
	BBClass			*clas;
	int				refs;
	
	BBException		*cause;
	BBString		*message;
};

extern BBClass bbRuntimeExceptionClass;

void bbRuntimeExceptionCtor(BBRuntimeException *ex);
void bbRuntimeExceptionDtor(BBRuntimeException *ex);

/** Gives the message of the runtime exception
 *
 */
BBString* bbRuntimeExceptionToString(const BBRuntimeException *ex);

/**
 *
 */
BBRuntimeException* bbRuntimeExceptionClone(const BBRuntimeException *ex);

//	API

/**
 *
 */
BBString* bbRuntimeExceptionGetMessage(const BBRuntimeException *ex);

/** Creates a new runtime exception object
 *
 * \param	message A description of the error
 * \param	cause The exception that initially caused the problem
 */
BBRuntimeException* bbRuntimeExceptionCreate(BBString *message, BBException *cause);

#define bbRuntimeError		brl_blitz_RuntimeError

/** Throws a BBRuntimeExeption with the given message
 *
 * \param	message The error message to display
 */
void bbRuntimeError(BBString *message);

BB_END_DECLS

#endif	// BLITZ_EX_RUNTIMEEXCEPTION_H
