Rem
A BlitzMax type that contains Abstract methods becomes abstract itself.
Abstract types are used to define interfaces that extending types must 
implement before they can be used to create new instances.

In the following code TShape is an abstract type in that you can not
create a TShape but anything extending a TShape must implement a Draw()
method.
End Rem

Type TShape
	Field xpos:Int
	Field ypos:Int
	
	Method Draw() Abstract
End Type

Type TCircle Extends TShape
	Field radius:Int
	
	Function Create:TCircle(x:Int, y:Int, radius:Int)
	
		Local circle:TCircle = New TCircle
		circle.xpos		= x
		circle.ypos		= y
		circle.radius	= radius
		Return circle
		
	End Function
	
	Method Draw()
		DrawOval(Self.xpos, Self.ypos, Self.radius, Self.radius)
	End Method
End Type

Type TRect Extends TShape
	Field width:Int
	Field height:Int
	
	Function Create:TRect(x:Int, y:Int, width:Int, height:Int)
	
		Local rect:TRect = New TRect
		rect.xpos	= x
		rect.ypos	= y
		rect.width	= width
		rect.height	= height
		Return rect
		
	End Function
	
	Method Draw()
		DrawRect(Self.xpos, Self.ypos, Self.width, Self.height)
	End Method
End Type

Local shape_list:TShape[4]
Local shape:TShape

shape_list[0] = TCircle.Create(200, 50, 50)
shape_list[1] = TRect.Create(300, 50, 40, 40)
shape_list[2] = TCircle.Create(400, 50, 50)
shape_list[3] = TRect.Create(200, 180, 250, 20)

Graphics(640, 480)

While Not KeyHit(KEY_ESCAPE)
	Cls()
	
	For shape:TShape = EachIn shape_list
		shape.draw()
	Next
	
	Flip()
Wend

End
