Rem
bbdoc:	Stores a sequence of elements
End Rem
Type Array Extends Object Abstract Final

	Rem
	bbdoc:	The number of elements the array holds
	about:	Read-only	
	End Rem
	Field length:Int

	Rem
	bbdoc:	Gives a short description of the array
	returns: A string describing the the array
	about:	The description string has the structure:
			<q>(Array) $TYPE[$DIMENSIONS]<q>
	End Rem
	Method toString:String()
	End Method
	
	Rem
	bbdoc:	Compares two arrays
	End Rem
	Method compare:Int(other:Array)
	End Method
	
	Rem
	bbdoc:	Checks if the array equals another one
	returns: <tt>True</tt> if the @other array equals this one, otherwise <tt>False</tt>.
	End Rem
	Method equals:Int(other:Array)
	End Method
	
	Rem
	bbdoc:	Clones the array
	returns: A new array with the same values
	about:	Primitive type array's (<tt>Int[]</tt>, <tt>Float[]</tt> etc) data will be copied by value.
			The data of complex type array's (<tt>Object[]</tt>, <tt>String[]</tt> etc) however, will be copied by reference.
	End Rem
	Method clone:Array()
	End Method



	Rem
	bbdoc:	States that an element or array could not be found
	about:	Used by #find, #findInt, #findLong, #findFloat, #findDouble, #findObject and #findString, 
	End Rem
	Const NOT_FOUND:Int = -1

	Rem
	bbdoc:	Finds the first occurance of a subarray in the array
	returns: The position of the first matching subarray, or #NOT_FOUND.
	End Rem
	Method find:Int(subarray:Array, start_index:Int = 0)
	End Method

	Rem
	bbdoc:	Finds the first occurance of an <tt>Int</tt> value in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findInt:Int(number:Int, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of a <tt>Long</tt> value in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findLong:Int(number:Long, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of a <tt>Float</tt> value in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findFloat:Int(number:Float, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of a <tt>Double</tt> value in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findDouble:Int(number:Double, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of an object in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findObject:Int(obj:Object, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of a string in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findString:Int(str:String, start_index:Int = 0)
	End Method
	
	
	
	Rem
	bbdoc:	Checks if the array starts with another one
	returns: <tt>True</tt> if the array starts with @subarray, otherwise <tt>False</tt>.
	End Rem
	Method startsWith:Int(subarray:Array)
	End Method
	
	Rem
	bbdoc:	Checks if the array ends with another one
	returns: <tt>True</tt> if the array ends with @subarray, otherwise <tt>False</tt>.
	End Rem
	Method endsWith:Int(subarray:Array)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains another one.
	returns: <tt>True</tt> if the array contains the @subarray at least once, otherwise <tt>False</tt>.
	End Rem
	Method contains:Int(subarray:Array)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains an <tt>Int</tt> element with the given value.
	returns: <tt>True</tt> if the array contains the @number at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsInt:Int(number:Int)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains a <tt>Long</tt> element with the given value.
	returns: <tt>True</tt> if the array contains the @number at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsLong:Int(number:Long)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains a <tt>Float</tt> element with the given value.
	returns: <tt>True</tt> if the array contains the @number at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsFloat:Int(number:Float)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains a <tt>Double</tt> element with the given value.
	returns: <tt>True</tt> if the array contains the @number at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsDouble:Int(number:Double)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains an object.
	returns: <tt>True</tt> if the array contains the @obj at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsObject:Int(obj:Object)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains a string.
	returns: <tt>True</tt> if the array contains the @str at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsString:Int(str:String)
	End Method


	
	Rem
	bbdoc:	Reverses the array's elements
	returns: A reversed version of the array.
	about:	Empty or single element arrays are returned unaltered.
	End Rem
	Method reverse:Array()
	End Method
	
	Rem
	bbdoc:	Rotates the array's elements
	End Rem
	Method rotate:Array(count:Int = 1)
	End Method
	
	Rem
	bbdoc:	Sorts the array's elements
	End Rem
	Method sort(ascending:Int = 1)
	End Method
	
	
	
	Rem
	bbdoc:	Gives the array's dimensions
	returns: A <tt>Int</tt> array containing the size of each dimension
	End Rem
	Method dimensions:Int[]()
	End Method

End Type
