Rem
bbdoc:	Represents a sequence of characters
about:	Strings in BlitzMax are UCS-2 encoded and of fixed length.

		<p>Technically they are objects, but since they are built into the language
		they are more powerful than user-defined objects.</p>
End Rem
Type String Extends Object Abstract Final

	Rem
	bbdoc:	Contains the number of characters the string stores
	about:	Read-only
	End Rem
	Field length:Int
	
	Rem
	bbdoc:	Gives a reference to the string itself
	returns: The same string that called the method
	End Rem
	Method toString:String()
	End Method
	
	Rem
	bbdoc:	Compares two strings
	returns: <dl>
				<dt><tt>0</tt></dt>
					<dd>The strings are equal</dd>
				<dt>Distance between first non-matching character codes</dt>
					<dd>Different characters in the compared sequence (= shorter string's length)</dd>
				<dt>Difference in length</dt>	
					<dd>In case all compared characters were equal</dd>
			</dl>
	about:	<p class="bbdoc-note">When the @other string is </p>
			
			See #equals for a faster and cleaner check for equality.
	End Rem
	Method compare:Int(other:String)
	End Method

	Rem
	bbdoc:	Checks if the string equals another one.
	returns: <tt>True</tt> if the string equals the @other one, otherwise <tt>False</tt>.
	End Rem
	Method equals:Int(other:String)
	End Method

	Rem
	bbdoc:	Clones the string
	returns: A new string object with the same values
	about:	This is rarely makes sense, as reference-counting is performed automatically by the runtime
			and string functions do not affect existing strings (read-only).
			However, when working with extern C/C++ libraries or performing own algorithms that
			may change a string's content, this can be used.
	End Rem
	Method clone:String()
	End Method


	
	Rem
	bbdoc:	States that a string could not be found.
	about:	Used by #find, #findLast and #findAny
	End Rem
	Const NOT_FOUND:Int = -1
	
	Rem
	bbdoc:	Finds the first occurance of a substring in the string.
	returns: The position of the first matching substring, or #NOT_FOUND.
	End Rem
	Method find:Int(substring:String, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the last occurance of a substring in the string.
	returns: The position of the last matching substring, or #NOT_FOUND.
	End Rem
	Method findLast:Int(subtstring:String, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Find the first occurance of any of the given substrings in the string.
	returns: The position of the first matching substring, or #NOT_FOUND if none of them was found.
	End Rem
	Method findAny:Int(all_substrings:String[], start_index:Int = 0)
	End Method

	Rem
	bbdoc:	Finds all occurances of a substring in the string.
	returns: An array containing the positions of all found matches, or <tt>Null</tt>.
	End Rem
	Method findAll:Int[](substring:String, start_index:Int = 0)
	End Method
	
	
	
	
	Rem
	bbdoc:	Checks if the string starts with another one.
	returns: <tt>True</tt> if the string starts with @substring, otherwise <tt>False</tt>.
	End Rem
	Method startsWith:Int(substring:String)
	End Method
	
	Rem
	bbdoc:	Checks if the string ends with another one.
	returns: <tt>True</tt> if the string ends with @substring, otherwise <tt>False</tt>.
	End Rem
	Method endsWith:Int(substring:String)
	End Method

	Rem
	bbdoc:	Checks if the string contains another one.
	returns: <tt>True</tt> if the string contains the @substring at least once, otherwise <tt>False</tt>.
	End Rem
	Method contains:Int(substring:String)
	End Method
	
	Rem
	bbdoc:	Counts the number of times the string contains a substring.
	returns: The number of times @substring was found in the string.
	End Rem
	Method count:Int(substring:String)
	End Method
	
	
	Rem
	bbdoc:	States that both sides of the string should be affected.
	End Rem
	Const SIDE_BOTH:Int = 0
	
	Rem
	bbdoc:	States that only the left side of the string should be affected.
	End Rem
	Const SIDE_LEFT:Int = -1
	
	Rem
	bbdoc:	States that only the right side of the string should be affected.
	End Rem
	Const SIDE_RIGHT:Int = 1
	
	Rem
	bbdoc:	Removes whitespace from the string
	returns: A new string with the whitespace removed from the given side(s).
	End Rem
	Method Trim:String(side:Int = String.SIDE_BOTH)
	End Method

	Rem
	bbdoc:	Pads the string to the given length
	returns: A new string padded to given @length with spaces, or the string itself.
	End Rem
	Method pad:String(length:Int, side:Int = String.SIDE_BOTH)
	End Method


	Rem
	bbdoc:	Replaces all occurances of a substring in the string
	returns: A new string with all occurances of @substring replaced with @replacement.
	about:	If no instance of @substring is found, the calling string object is returned unaltered.
	End Rem
	Method Replace:String(substring:String, replacement:String)
	End Method
	
	Rem
	bbdoc:	Splits the string into bits at every occurance of a separator.
	returns: An array containing all bits of the split string, or <tt>Null</tt>.
	End Rem
	Method split:String[](separator:String)
	End Method
	
	Rem
	bbdoc:	Joins strings together by inserting this string between them.
	returns: A new string containing all given @bits connected by this string, or <tt>Null</tt>.
	End Rem
	Method join:String(bits:String[])
	End Method
	
	Rem
	bbdoc:	Reverses the string.
	returns: A new string which contains the string's contents completely reversed.
	about:	Empty or one character strings are returned unaltered.
	End Rem
	Method reverse:String()
	End Method
	
	Rem
	bbdoc:	Rotates the string
	returns: A new string which contains the string's content rotated by the number of times given in @count.
	about:	The rotation is carried out following these rules for @count:
			<ul>
				<li><tt>positive</tt> Rotation to the right</li>
				<li><tt>negative</tt> Rotation to the left</li>
				<li><tt>0</tt> or the string's <tt>length</tt> No rotation</li>
			</ul>
	End Rem
	Method rotate:String(count:Int)
	End Method

	Rem
	bbdoc:	Repeats and concatenates the string serveral times.
	returns: A new string with the contents concatenated for the given amount of times, or <tt>Null</tt>.
	about:	If @count is <tt>1</tt> the string is returned unaltered.
	End Rem
	Method times:String(count:Int)
	End Method
	
	Rem
	bbdoc:	Gives a part of the string
	returns: A substring containing a part of the string, or <tt>Null</tt>.
	End Rem
	Method sub:String(start_index:Int, length:Int)
	End Method
	

	Rem
	bbdoc:	Converts the string to lowercase
	returns: A new string with all characters in lower case
	End Rem
	Method toLower:String()
	End Method
	
	Rem
	bbdoc:	Converts the string to uppercase
	returns: A new string with all characters in upper case
	End Rem
	Method toUpper:String()
	End Method
	
	Rem
	bbdoc:	Checks if the string is lowercase only
	returns: <tt>True</tt> if all characters are lowercase, otherwise <tt>False</tt>.
	End Rem
	Method isLower:Int()
	End Method
	
	Rem
	bbdoc:	Checks if the string is uppercase only
	returns: <tt>True</tt> if all characters are uppcase, otherwise <tt>False</tt>.
	End Rem
	Method isUpper:Int()
	End Method
	
	Rem
	bbdoc:	Checks if the string contains only ASCII characters.
	returns: <tt>True</tt> if no character is out of ASCII range, otherwise <tt>False</tt>.
	End Rem
	Method isASCII:Int()
	End Method
	
	

	Rem
	bbdoc:	Creates a string from an integer
	returns: A new string containing the <b>number</b>'s value.
	End Rem
	Function FromInt:String(number:Int)
	End Function
	
	Rem
	bbdoc:	Creates a string from a long integer
	returns: A new string containing the <b>number</b>'s value.
	End Rem
	Function FromLong:String(number:Long)
	End Function
	
	Rem
	bbdoc:	Creates a string from a single-precision floating point value
	returns: A new string containing the <b>number</b>'s value.
	End Rem
	Function FromFloat:String(number:Float)
	End Function
	
	Rem
	bbdoc:	Creates a string from a double-precision floating point value
	returns: A new string containing the <b>number</b>'s value.
	End Rem
	Function FromDouble:String(number:Double)
	End Function
	
	Rem
	bbdoc:	Converts the string to an integer
	returns: The parsed value, or <tt>0</tt> in case none could be found.
	End Rem
	Method toInt:Int()
	End Method
	
	Rem
	bbdoc:	Converts the string to an integer
	returns: The parsed value, or <tt>0</tt> in case none could be found.
	End Rem
	Method toLong:Long()
	End Method
	
	Rem
	bbdoc:	Converts the string to an integer
	returns: The parsed value, or <tt>0</tt> in case none could be found.
	End Rem
	Method toFloat:Float()
	End Method
	
	Rem
	bbdoc:	Converts the string to an integer
	returns: The parsed value or <tt>0</tt> in case none could be found.
	End Rem
	Method toDouble:Double()
	End Method
	
	
	
	Rem
	bbdoc:	Creates a new string from a sequence of 8 bit character data
	returns: A new string containing the data in @bytes or <tt>Null</tt>.
	End Rem
	Function FromBytes:String(bytes:Byte Ptr, size:Int)
	End Function
	
	Rem
	bbdoc:	Creates a new string from a sequence of 16 bit character data
	returns: A new string containing the data in @shorts or <tt>Null</tt>.
	End Rem
	Function FromShorts:String(shorts:Short Ptr, size:Int)
	End Function
	
	Rem
	bbdoc:	Creates a new string from a sequence of null-terminated 8 bit character data.
	returns: A new string containing the data in @c_string or <tt>Null</tt>.
	End Rem
	Function FromCString:String(c_string:Byte Ptr)
	End Function
	
	Rem
	bbdoc:	Creates a new string from a sequence of null-terminated 16 bit character data.
	returns: A new string containing the data in @w_string or <tt>Null</tt>.
	End Rem
	Function FromWString:String(w_string:Short Ptr)
	End Function
	
	Rem
	bbdoc:	Creates a new string from a sequence of UTF-8 encoded character data
	returns: A new string containing the data in @utf8_string or <tt>Null</tt>.
	End Rem
	Function FromUTF8String:String(utf8_string:Byte Ptr)
	End Function
	
	Rem
	bbdoc:	Converts the string to a sequence of null-terminated 8 bit character data.
	returns: A newly allocated block of memory containing the string's data.
	End Rem
	Method toCString:Byte Ptr()
	End Method
	
	Rem
	bbdoc:	Converts the string to a sequence of null-terminated 16 bit character data.
	returns: A newly allocated block of memory containing the string's data.
	End Rem
	Method toWString:Short Ptr()
	End Method
	
	Rem
	bbdoc:	Converts the string to a sequence of null-terminated UTF-8 encoded character data.
	returns: A newly allocated block of memory containing the string's data.
	End Rem
	Method toUTF8String:Byte Ptr()
	End Method

End Type