'BlitzMax keyword definitions

Rem
bbdoc:	Set strict mode
keyword: "Strict"
about:	See the <a href="../../../../doc/bmxlang/compatibility.html">BlitzMax Language Reference</a> for more information on Strict mode programming.
		
		<b>Use #SuperStrict instead.</b>
End Rem

Rem
bbdoc:	Set SuperStrict mode
keyword:"SuperStrict"
about:	Forces clean programming practices (e.g. all variables need to be declared with a type).

		<p>Just like #Strict this statement has to appear at the top of the source file
		which it should affect. Only #Framework and #Import statements may be used before it.</p>

		<p>When including files, only the main one has to set the mode via <tt>SuperStrict</tt>.
		However if you use #Import to add a file, it will be handled as an independently compiled
		file and therefore has to declare itself <tt>SuperStrict</tt>, too.</p>
End Rem

Rem
bbdoc:	Ends program execution
keyword: "End"
End Rem

Rem
bbdoc:	Begins a remark block
keyword: "Rem"
End Rem

Rem
bbdoc:	Ends a remark block
keyword: "EndRem"
End Rem

Rem
bbdoc:	Constant integer of value 1
keyword: "True"
End Rem

Rem
bbdoc:	Constant integer of value 0
keyword: "False"
End Rem

Rem
bbdoc:	Constant Pi value: 3.1415926535897932384626433832795
keyword: "Pi"
End Rem

Rem
bbdoc:	Get Default Null value
keyword: "Null"
End Rem

Rem
bbdoc:	Unsigned 8 bit integer type
keyword: "Byte"
about:	For the numeric limits see #BYTE_MIN and #BYTE_MAX
End Rem

Rem
bbdoc:	Unsigned 16 bit integer type
keyword: "Short"
about:	For the numeric limits see #SHORT_MIN and #SHORT_MAX
End Rem

Rem
bbdoc:	Signed 32 bit integer type
keyword: "Int"
about:	For the numeric limits see #INT_MIN and #INT_MAX

		<p>This is the most commonly used type throughout the runtime.
		It is usually the fastest one and is sufficient for most integer numbers.</p>
End Rem

Rem
bbdoc:	Signed 64 bit integer type
keyword: "Long"
about:	For the numeric limits see #LONG_MIN and #LONG_MAX
End Rem

Rem
bbdoc:	32 bit Floating point type
keyword: "Float"
about:	For the numeric limits see #FLOAT_MIN and #FLOAT_MAX
End Rem

Rem
bbdoc:	64 bit floating point type
keyword: "Double"
about:	For the numeric limits see #DOUBLE_MIN and #DOUBLE_MAX
End Rem

'Rem
'bbdoc:	String Type
'keyword: "String"
'End Rem

'Rem
'bbdoc:	Object Type
'keyword: "Object"
'End Rem

Rem
bbdoc:	Composite type specifier for 'by reference' types
keyword: "Var"
End Rem

Rem
bbdoc:	Composite type specifier for pointer types
keyword: "Ptr"
End Rem

Rem
bbdoc:	Begins a conditional block.
keyword: "If"
End Rem

Rem
bbdoc:	Optional separator between the condition and associated code in an If statement.
keyword: "Then"
End Rem

Rem
bbdoc:	Provides the ability for an <tt>If-Then</tt> construct to execute a second block of code when the <tt>If</tt> condition is <tt>False</tt>.
keyword: "Else"
End Rem

Rem
bbdoc:	Provides the ability to test and execute a section of code if the initial condition failed.
keyword: "ElseIf"
End Rem

Rem
bbdoc:	Marks the End of an <tt>If-Then</tt> block.
keyword: "EndIf"
End Rem

Rem
bbdoc:	Marks the start of a loop that uses an iterator to execute a section of code repeatedly.
keyword: "For"
End Rem

Rem
bbdoc:	Followed by a constant which is used to calculate when to exit a <tt>For-Next</tt> loop.
keyword: "To"
End Rem

Rem
bbdoc:	Specifies an optional constant that is used to increment the <tt>For</tt> iterator.
keyword: "Step"
End Rem

Rem
bbdoc:	Ends a <tt>For</tt> block
keyword: "Next"
End Rem

Rem
bbdoc:	Iterates through an array or collection in a <tt>For</tt> loop
keyword: "EachIn"
about:	Own <tt>Type</tt>s can make use of this by implementing the #IEnumerable interface.
End Rem

Rem
bbdoc:	Executes a block of code while a condition is <tt>True</tt>
keyword: "While"
End Rem

Rem
bbdoc:	Ends a <tt>While</tt> block
keyword: "Wend"
End Rem

Rem
bbdoc: Ends a <tt>While</tt> block
keyword: "EndWhile"
about:	Very unusal. Use #Wend instead to make code easier to read by others.
End Rem

Rem
bbdoc:	Executes a block of code until a termination condition is met
keyword: "Repeat"
about:	This kind of loop can be executed until a certain condition is met by using #Until,
		or run indefinitely with the #Forever statement.
End Rem

Rem
bbdoc:	Conditionally continue a <tt>Repeat</tt> block
keyword: "Until"
about:	Instead of specifying a condition that will never be met, use #Forever to
		execute the loop indefinitely.
End Rem

Rem
bbdoc:	Continues a #Repeat loop forever
keyword: "Forever"
about:	There should be at least one check for a condition to exit the loop within the block.
End Rem

Rem
bbdoc:	Begins a Select block
keyword: "Select"
End Rem

Rem
bbdoc:	Ends a Select block
keyword: "EndSelect"
End Rem

Rem
bbdoc:	Conditional code inside a <tt>Select</tt> block
keyword: "Case"
End Rem

Rem
bbdoc:	Contains the default code inside a <tt>Select</tt> block
keyword: "Default"
about:	This block of code is called in case none of the given #Case conditions is a match.
End Rem

Rem
bbdoc:	Exits the enclosing loop
keyword: "Exit"
End Rem

Rem
bbdoc:	Continues execution of the enclosing loop
keyword: "Continue"
End Rem

Rem
bbdoc:	Declares a constant
keyword: "Const"
about:	A constants can be of any primitive type (<tt>Byte</tt>, <tt>Short</tt>, <tt>Int</tt>, <tt>Long</tt>
		<tt>Float</tt> or <tt>Double</tt>) or <tt>String</tt>.
		
		<p>While possible it is rarely practical to define a <tt>Object</tt> constant.</p>
End Rem

Rem
bbdoc:	Declares a local variable
keyword: "Local"
about:	Local variables are only available in their respective scope.
		That means code outside of their immediate scope can not "see" (access) them.
		
		<p class="bbdoc-note">If there is a #global variable with the same name,
		the newly declared local variable hides it in the current block.</p>
End Rem

Rem
bbdoc:	Declares a global variable
keyword: "Global"
End Rem

Rem
bbdoc:	Declares a class instance variable
keyword: "Field"
End Rem

Rem
bbdoc:	Begins a <tt>Function</tt> declaration
keyword: "Function"
End Rem

Rem
bbdoc:	Ends a <tt>Function</tt> declaration
keyword: "EndFunction"
End Rem

Rem
bbdoc:	Returns from a Function
keyword: "Return"
End Rem

Rem
bbdoc:	Begins a user defined Type declaration
keyword: "Type"
End Rem

Rem
bbdoc:	Ends a user defined Type declaration
keyword: "EndType"
End Rem

Rem
bbdoc:	Specifies user defined Type supertype
keyword: "Extends"
End Rem

Rem
bbdoc:	Begins a Method declaration
keyword: "Method"
End Rem

Rem
bbdoc:	Ends a Method declaration
keyword: "EndMethod"
End Rem

Rem
bbdoc:	Denotes a <tt>Type</tt>, <tt>Method</tt> or Type <tt>Function</tt> as abstract
keyword: "Abstract"
End Rem

Rem
bbdoc:	Denotes a <tt>Type</tt>, <tt>Method</tt> or Type <tt>Function</tt> as abstract
keyword: "Final"
End Rem

Rem
bbdoc:	Creates an instance of a user defined <tt>Type</tt>
keyword: "New"
End Rem

Rem
bbdoc:	References to this Method's Object instance
keyword: "Self"
End Rem

Rem
bbdoc:	References to the Super Type Object instance
keyword: "Super"
End Rem

Rem
bbdoc:	Reserved for future expansion
keyword: "Delete"
End Rem

Rem
bbdoc:	Releases an integer Object handle
keyword: "Release"
End Rem

Rem
bbdoc:	Makes a Constant, Global variable or Function accessible from outside the current source file (Default)
keyword: "Public"
End Rem

Rem
bbdoc:	Makes a Constant, Global variable or Function only accessible from within the current source file
keyword: "Private"
End Rem

Rem
bbdoc:	Marks the beginning of an external list of <tt>Function</tt> declarations
keyword: "Extern"
End Rem

Rem
bbdoc:	Marks the end of an <tt>Extern</tt> section
keyword: "EndExtern"
End Rem

Rem
bbdoc:	Declares a module scope and identifier
about:
See the <a href=../../../../doc/bmxlang/modules.html>BlitzMax Language Reference</a> for more information on BlitzMax Modules.
keyword: "Module"
End Rem

Rem
bbdoc:	Defines a module's properties
keyword: "ModuleInfo"
End Rem

Rem
bbdoc:	Embeds a data file in the application file
keyword: "Incbin"
End Rem

Rem
bbdoc:	Get the start memory address of an embedded data file
keyword: "IncbinPtr"
End Rem

Rem
bbdoc:	Get the length of an embedded data file
keyword: "IncbinLen"
End Rem

Rem
bbdoc:	Includes the specified file into the file being compiled.
keyword: "Include"
about:	Basically <tt>Include</tt> insert the content of the file given as a parameter into the
		the file being compiled. 
End Rem

Rem
bbdoc:	Builds the application with only the module specified rather than all modules installed.
keyword: "Framework"
about:	Before releasing an application you should always use <tt>Framework</tt> and <tt>Import</tt>
		to include only the necessary modules. This keeps the application's size at a minimum.
End Rem

Rem
bbdoc:	Import declarations from a module or source file
keyword: "Import"
End Rem

Rem
bbdoc:	Throws a #RuntimeError if the given condition is <tt>False</tt>
keyword: "Assert"
End Rem

Rem
bbdoc:	Transfers program flow to specified label
keyword: "Goto"
about:	Goto is infamous for leading to so-called "spaghetti code". This means code that is hard to follow
		because it "jumps" back and forth between commands in a very unstructured way. Use <tt>Goto</tt> very
		carefully, if at all.
		
		<p>Not available in #Strict or #SuperStrict mode. Therefore very uncommon.</p>
End Rem

Rem
bbdoc:	Begins an exception-catching <tt>Try</tt block
keyword: "Try"
End Rem

Rem
bbdoc:	Ends a <tt>Try</tt> block
keyword: "EndTry"
End Rem

Rem
bbdoc:	Catches an exception object in a <tt>Try</tt> block
keyword: "Catch"
about:	Try to be as specific as possible when declaring the exception's type
		to avoid catching an exception that was not meant to be handled by this <tt>Try</tt> block.
End Rem

Rem
bbdoc:	Throws an exception object to the enclosing <tt>Try</tt> block
keyword: "Throw"
End Rem

Rem
bbdoc:	Defines classic BASIC style data
keyword: "DefData"
End Rem

Rem
bbdoc:	Read classic BASIC style data
keyword: "ReadData"
End Rem

Rem
bbdoc:	Restores classic BASIC style data
keyword: "RestoreData"
End Rem

Rem
bbdoc:	Conditional 'And' binary operator
keyword: "And"
End Rem

Rem
bbdoc:	Conditional 'Or' binary operator
keyword: "Or"
End Rem

Rem
bbdoc:	Conditional 'Not' binary operator
keyword: "Not"
End Rem

Rem
bbdoc:	Bitwise 'Shift left' binary operator
keyword: "Shl"
End Rem

Rem
bbdoc:	Bitwise 'Shift right' binary operator
keyword: "Shr"
End Rem

Rem
bbdoc:	Bitwise 'Shift arithmetic right' binary operator
keyword: "Sar"
End Rem

Rem
bbdoc:	Number of characters in a <tt>String</tt> or elements in an <tt>Array</tt>
keyword: "Len"
about:	Alternate method for directly accessing the string's respectively array's <tt>length</tt> property.
End Rem

Rem
bbdoc:	Numeric 'absolute value' unary operator
keyword: "Abs"
End Rem

Rem
bbdoc:	Numeric 'modulus' or 'remainder' binary operator
keyword: "Mod"
End Rem

Rem
bbdoc:	Numeric 'sign' unary operator
keyword: "Sgn"
End Rem

Rem
bbdoc:	Numeric 'minimum' builtin function
returns: The lesser of the two numeric arguments
keyword: "Min"
End Rem

Rem
bbdoc:	Numeric 'maximum' builtin function
returns: The larger of the two numeric arguments
keyword: "Max"
End Rem

Rem
bbdoc:	Find the address of a variable
keyword: "Varptr"
End Rem

Rem
bbdoc:	Size, in bytes, occupied by a variable, string, array or object
keyword: "SizeOf"
End Rem

Rem
bbdoc:	Get character value of the first character of a string
keyword: "Asc"
End Rem

Rem
bbdoc:	Creates a string of length 1 with a character code
keyword: "Chr"
End Rem
